#ifndef __G3D_CONTEXT_H__
#define __G3D_CONTEXT_H__

#include <g3d/types.h>

G_BEGIN_DECLS

/**
 * g3d_context_new:
 *
 * Create a new context. This initializes the library (and also the
 * plugin system so this has not to be done seperately).
 *
 * Returns: a valid context, or NULL on failure.
 */

G3DContext *g3d_context_new(void);

/**
 * g3d_context_free:
 *
 * Cleans up the context and the plugin system and frees all reserved
 * memory.
 */

void g3d_context_free(G3DContext *context);

/**
 * g3d_context_set_bgcolor:
 * @context: a valid context
 * @r: red component (range: 0.0 .. 1.0)
 * @g: green component (range: 0.0 .. 1.0)
 * @b: green component (range: 0.0 .. 1.0)
 *
 * Try to set the background color. This will call a function registered
 * with @g3d_context_set_set_bgcolor_func.
 *
 * Returns: TRUE on success, FALSE else
 */

gboolean g3d_context_set_bgcolor(G3DContext *context,
	gfloat r, gfloat g, gfloat b, gfloat a);

/**
 * g3d_context_update_interface:
 * @context: a valid context.
 *
 * Try to update the interface. This will call a function registered with
 * @g3d_context_set_update_interface_func.
 *
 * Returns: TRUE on success, FALSE else
 */

gboolean g3d_context_update_interface(G3DContext *context);

gboolean g3d_context_update_progress_bar(G3DContext *context,
	gfloat percentage, gboolean visibility);

/* set callback functions */
void g3d_context_set_set_bgcolor_func(G3DContext *context,
	G3DSetBgColorFunc func, gpointer user_data);

G_END_DECLS

#endif /* __G3D_CONTEXT_H__ */
